/*
 * ui.h - hlogin(1) user interface functions
 *
 * Copyleft (C) 2022 ~keith <keith@keithhacks.cyou>
 *
 * This file is part of hlogin.
 *
 * hlogin is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, version 3.
 *
 * hlogin is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with hlogin. If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef __UI_H
#define __UI_H

#include <ncurses.h>
#include <panel.h>

extern int term_rows, term_cols;

void ui_init();

#define UI_IT_TEXT   0
#define UI_IT_BUTTON 1
#define UI_IT_INPUT  2

#define UI_HIDDEN  0x1
#define UI_NOFOCUS 0x2
#define UI_CANFREE 0x4

struct ui_item {
	char   type;
	short  flags;
	int    x;
	int    y;
	int    cols;
	int    rows;
	char  *str;
	void  *data;

	struct ui_win  *win;
	struct ui_item *prev;
	struct ui_item *next;
};

#define UI_MT_ACCEPT   0 /* Accepting action, e.g. selecting a button */
#define UI_MT_REJECT   1 /* Rejecting action, e.g. pressing Escape */
#define UI_MT_FOCUS    2 /* Focus changed */
#define UI_MT_MODIFY   3 /* Value modified, e.g. typing in a text field */
#define UI_MT_VIEWPORT 4 /* Viewport shifted, e.g. scrolling a list */

struct ui_msg {
	char  type;
	char  cparam;
	short wparam;
	int   lparam;
	void *hparam;

	struct ui_win  *win;
	struct ui_item *item;
};

struct ui_win {
	WINDOW *win;
	PANEL  *panel;
	short   flags;
	int     x;
	int     y;
	int     cols;
	int     rows;

	struct ui_item *first;
	struct ui_item *focus;
	struct ui_item *last;
};

struct ui_win * ui_win_new(short flags, int x, int y, int cols, int rows);
void ui_win_destroy(struct ui_win *w);
void ui_win_focus(struct ui_win *w);
bool ui_win_focusp(struct ui_win *w);
struct ui_win * ui_win_get_focus();
void ui_win_focus_rotate(bool upwards);
void ui_win_focus_swap(int depth);

struct ui_item * ui_item_new(struct ui_win *w, char type, short flags);
void ui_item_destroy(struct ui_item *item);
void ui_item_reorder(struct ui_item *item, struct ui_item *after);
void ui_item_navigate(struct ui_item *item, bool backwards);

void draw_outline(WINDOW *win, int height, int width, int y, int x, attr_t light, attr_t shadow);

// string allocation size constraints
#define ASTR_START 128
#define ASTR_LIMIT 16384

#endif // __UI_H
